package recipe

import (
	"fmt"
	"os"
	"path"

	"coopcloud.tech/abra/pkg/envfile"
	"coopcloud.tech/abra/pkg/formatter"
)

func (r Recipe) SampleEnv() (map[string]string, error) {
	sampleEnv, err := envfile.ReadEnv(r.SampleEnvPath)
	if err != nil {
		return sampleEnv, fmt.Errorf("unable to discover .env.sample for %s", r.Name)
	}
	return sampleEnv, nil
}

// GetReleaseNotes prints release notes for the recipe version
func (r Recipe) GetReleaseNotes(version string) (string, error) {
	if version == "" {
		return "", nil
	}

	fpath := path.Join(r.Dir, "release", version)

	if _, err := os.Stat(fpath); !os.IsNotExist(err) {
		releaseNotes, err := os.ReadFile(fpath)
		if err != nil {
			return "", err
		}

		title := formatter.BoldStyle.Render(fmt.Sprintf("%s release notes:", version))
		withTitle := fmt.Sprintf("%s\n%s\n", title, releaseNotes)

		return withTitle, nil
	}

	return "", nil
}
