#!/usr/bin/env bash

setup_file(){
  load "$PWD/tests/integration/helpers/common"
  _common_setup
  _add_server
  _new_app
}

teardown_file(){
  _rm_app
  _rm_server
}

setup(){
  load "$PWD/tests/integration/helpers/common"
  _common_setup
}

teardown(){
  _reset_recipe
  _undeploy_app
}

@test "validate app argument" {
  run $ABRA app undeploy
  assert_failure

  run $ABRA app undeploy DOESNTEXIST
  assert_failure
}

# bats test_tags=slow
@test "ensure recipe up to date if no --offline" {
  _deploy_app

  wantHash=$(_get_n_hash 3)

  run git -C "$ABRA_DIR/recipes/$TEST_RECIPE" reset --hard HEAD~3
  assert_success

  assert_equal $(_get_current_hash) "$wantHash"

  run $ABRA app undeploy "$TEST_APP_DOMAIN" --no-input
  assert_success

  assert_equal $(_get_head_hash) $(_get_current_hash)
}

# bats test_tags=slow
@test "ensure recipe not up to date if --offline" {
  _deploy_app

  _ensure_env_version "0.1.0+1.20.0"
  latestRelease=$(_latest_release)

  run git -C "$ABRA_DIR/recipes/$TEST_RECIPE" tag -d "$latestRelease"
  assert_success

  run $ABRA app undeploy "$TEST_APP_DOMAIN" --no-input --offline
  assert_success

  run git -C "$ABRA_DIR/recipes/$TEST_RECIPE" tag -l
  refute_output --partial "$latestRelease"
}

@test "error if not deployed" {
  run $ABRA app undeploy "$TEST_APP_DOMAIN"
  assert_failure
  assert_output --partial 'is not deployed'
}

# bats test_tags=slow
@test "do not bail if unstaged changes (only query runtime)" {
  _deploy_app

  run bash -c "echo foo >> $ABRA_DIR/recipes/$TEST_RECIPE/foo"
  assert_success
  assert_exists "$ABRA_DIR/recipes/$TEST_RECIPE/foo"

  run $ABRA app undeploy "$TEST_APP_DOMAIN" --no-input
  assert_success

  run rm -rf "$ABRA_DIR/recipes/$TEST_RECIPE/foo"
  assert_not_exists "$ABRA_DIR/recipes/$TEST_RECIPE/foo"
}

# bats test_tags=slow
@test "undeploy app" {
  run $ABRA app deploy "$TEST_APP_DOMAIN" --no-input
  assert_success

  run $ABRA app undeploy "$TEST_APP_DOMAIN" --no-input
  assert_success
}

# bats test_tags=slow
@test "undeploy and prune" {
  _deploy_app

  run $ABRA app undeploy "$TEST_APP_DOMAIN" --no-input --prune
  assert_success
}

# bats test_tags=slow
@test "undeploy chaos deployment" {
  run $ABRA app deploy "$TEST_APP_DOMAIN" --no-input --chaos

  run $ABRA app undeploy "$TEST_APP_DOMAIN" --no-input
  assert_success
  
  # NOTE(d1): ensure chaos undeploy
  assert_output --partial ${_get_current_hash:0:8}
  refute_output --partial 'false'
}
